import 'package:flutter/material.dart';
import 'package:flutter_hooks/flutter_hooks.dart' as flutter_hooks;
import 'package:kohana/src/ui/lifecycle.dart';
import 'package:rxdart/rxdart.dart';

// ignore: must_be_immutable
abstract class StatefulView<T> extends flutter_hooks.HookWidget {
  _Fields _managedFields;
  ValueNotifier<int> stateTrigger;
  T $;

  T setup();

  CompositeSubscription get disposeBag => _managedFields.disposeBag;

  BuildContext get context => _managedFields.context;

  bool get mounted => _managedFields.mounted;

  get managedField => flutter_hooks.useMemoized;

  List<Dispose> _onUnmounted = [];
  List<Dispose> _onMounted = [];

  void onMounted(Dispose dispose) => _onMounted.add(dispose);

  void onUnmounted(Dispose dispose) => _onUnmounted.add(dispose);

  StatefulView({Key key}) : super(key: key);

  ///use [buildWidget] method instead [build]]
  Widget buildWidget(BuildContext context);

  @override
  Widget build(BuildContext context) {
    runHooks();
    return buildWidget(context);
  }

  //explicitly update state when working with non reactive data
  void setState([VoidCallback fn]) {
    if (fn != null) fn();
    ++stateTrigger.value;
  }

  /// https://reactjs.org/docs/hooks-effect.html
  @mustCallSuper
  void runHooks() {
    _handleManagedFields();
    flutter_hooks.useEffect(_handleLifecycleEvents, []);
  }

  void _handleManagedFields() {
    final context = flutter_hooks.useContext();
    stateTrigger = flutter_hooks.useState(0);
    _managedFields = managedField(() => _Fields(context));
    _managedFields.context = context;
    $ = flutter_hooks.useMemoized(() => setup(), []);
  }

  void _componentDidMount() async {
    if ($ is LifecycleAware) {
      var model = $ as LifecycleAware;
      await model.start();
    }
    _managedFields.mounted = true;
    for (var $ in _onMounted) {
      await $();
    }
    componentDidMount();
  }

  void _componentWillUnmount() async {
    for (var $ in _onUnmounted) {
      await $();
    }
    if ($ is LifecycleAware) {
      var model = $ as LifecycleAware;
      await model.dispose();
    }
    _managedFields.mounted = false;
    disposeBag.clear();
    componentWillUnmount();
  }

  Dispose _handleLifecycleEvents() {
    _componentDidMount();
    return _componentWillUnmount;
  }

  void componentDidMount() async {}

  void componentWillUnmount() async {}
}

class _Fields {
  final CompositeSubscription disposeBag = CompositeSubscription();
  var mounted = false;
  BuildContext context;

  _Fields(this.context);
}
